package eu.lucazanini.service;

import java.lang.ref.WeakReference;
import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.RadioGroup;
import android.widget.TextView;
import eu.lucazanini.service.MyBoundService.LocalBinder;

public class MainActivity extends Activity implements OnClickListener {

    /**
     * Instances of static inner classes do not hold an implicit reference to
     * their outer class.
     */
    private static class MyHandler extends Handler {
	private final WeakReference<MainActivity> mActivity;

	public MyHandler(MainActivity activity) {
	    mActivity = new WeakReference<MainActivity>(activity);
	}

	@Override
	public void handleMessage(Message msg) {
	    MainActivity activity = mActivity.get();
	    if (activity != null) {
		Bundle b = msg.getData();
		String key = b.getString("timeKey");
		activity.tv.setText(key);
	    }
	}
    }

    private final MyHandler handler = new MyHandler(this);

    private MyBoundService mBoundService;
    /** Defines callbacks for service binding, passed to bindService() */
    private ServiceConnection mConnection = new ServiceConnection() {

	@Override
	public void onServiceConnected(ComponentName className, IBinder service) {
	    // This is called when the connection with the service has been
	    // established, giving us the service object we can use to
	    // interact with the service. Because we have bound to a explicit
	    // service that we know is running in our own process, we can
	    // cast its IBinder to a concrete class and directly access it.
	    mBoundService = ((LocalBinder) service).getService();

	    // LocalBinder binder = (LocalBinder) service;
	    // mBoundService = binder.getService();

	    Thread t = new Thread(separateThread);
	    t.start();

	}

	@Override
	public void onServiceDisconnected(ComponentName className) {
	    // This is called when the connection with the service has been
	    // unexpectedly disconnected -- that is, its process crashed.
	    // Because it is running in our same process, we should never
	    // see this happen.
	    mBoundService = null;
	}

    };

    private Runnable separateThread = new Runnable() {
	@Override
	public void run() {
	    mBoundService.test(tv, handler);
	    doUnbindService();
	}

    };

    private final String TAG = this.getClass().getName();
    private boolean mIsBound = false;
    private CheckBox cb;
    private RadioGroup rg;
    private TextView tv;

    private void doBindService() {
	// Establish a connection with the service. We use an explicit
	// class name because we want a specific service implementation that
	// we know will be running in our own process (and thus won't be
	// supporting component replacement by other applications).
	bindService(new Intent(MainActivity.this, MyBoundService.class),
		mConnection, Context.BIND_AUTO_CREATE);
	mIsBound = true;
    }

    private void doUnbindService() {
	if (mIsBound) {
	    // Detach our existing connection.
	    unbindService(mConnection);
	    mIsBound = false;
	}
    }

    @Override
    public void onClick(View v) {
	Intent intent;
	switch (v.getId()) {
	case R.id.btn_started:
	    Log.d(TAG, "started service click");

	    intent = new Intent(MainActivity.this, MyStartedService.class);
	    Bundle extras = new Bundle();
	    extras.putBoolean("onDestroy", cb.isChecked());
	    if (rg.getCheckedRadioButtonId() == R.id.radio2) {
		extras.putString("return", "START_NOT_STICKY");
	    } else {
		extras.putString("return", "START_STICKY");
	    }
	    intent.putExtras(extras);
	    startService(intent);
	    break;
	case R.id.btn_bounded:
	    Log.d(TAG, "bound service click");

	    doBindService();
	    break;
	}
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
	super.onCreate(savedInstanceState);
	setContentView(R.layout.activity_main);

	tv = (TextView) findViewById(R.id.lblOutput);

	Button btn1 = (Button) findViewById(R.id.btn_started);
	btn1.setOnClickListener(this);

	Button btn2 = (Button) findViewById(R.id.btn_bounded);
	btn2.setOnClickListener(this);

	cb = (CheckBox) findViewById(R.id.checkBox1);

	rg = (RadioGroup) findViewById(R.id.radioGroup1);

	// Bind to Bound Service
	Intent intent = new Intent(this, ServiceConnection.class);
	bindService(intent, mConnection, Context.BIND_AUTO_CREATE);

    }

    @Override
    protected void onDestroy() {
	super.onDestroy();
	doUnbindService();
    }

}
